package be.iminds.ilabt.jfed.lowlevel.api;

import be.iminds.ilabt.jfed.lowlevel.GeniAMResponseCode;
import org.testng.annotations.Test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class AbstractGeniAggregateManagerTest {
    @Test
    public void testIsNotEnoughFreeResources() throws Exception {
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getValue()).thenReturn("Could not map to resources: Precheck failure");
        when(reply.getRawValue()).thenReturn("Could not map to resources: Precheck failure");
        when(reply.getOutput()).thenReturn("  *** No available physical nodes of type SERVER5P found (1 requested)\n");
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.GENIRESPONSE_ERROR);
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(true));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(false));
     }
    @Test
    public void testIsNotEnoughFreeResources2() throws Exception {
        //same as testIsNotEnoughFreeResources() but without value, should also work!
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getValue()).thenReturn(null);
        when(reply.getRawValue()).thenReturn("Could not map to resources: Precheck failure");
        when(reply.getOutput()).thenReturn("  *** No available physical nodes of type SERVER5P found (1 requested)\n");
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.GENIRESPONSE_ERROR);
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(true));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(false));
     }
    @Test
    public void testIsNotEnoughFreeResourcesPerformLte() throws Exception {
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getValue()).thenReturn(null);
        when(reply.getRawValue()).thenReturn("");
        when(reply.getOutput()).thenReturn("Too Big: insufficient resources to fulfill request");
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.GENIRESPONSE_TOOBIG);
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(true));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(false));
     }
    @Test
    public void testIsNotEnoughFreeResourcesExogeni() throws Exception {
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getValue()).thenReturn(null);
        when(reply.getRawValue()).thenReturn(null);
        when(reply.getOutput()).thenReturn("Embedding workflow ERROR: 2:Not enough resources in domain: http://geni-orca.renci.org/owl/uvanlvmsite.rdf#uvanlvmsite/Domain/vm:vm; Requested:1;resource pool=0.\\n Please see https://geni-orca.renci.org/trac/wiki/orca-errors for possible solutions.");
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.getByCode(2));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(true));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(false));
     }
    @Test
    public void testIsNotEnoughFreeResourcesLogATec() throws Exception {
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getValue()).thenReturn(null);
        when(reply.getRawValue()).thenReturn(null);
        when(reply.getOutput()).thenReturn("insufficient resources available");
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.GENIRESPONSE_REFUSED);
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(true));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(false));
     }

    //TODO: add some more examples of isNotEnoughFreeResources()


    @Test
    public void testIsTermsAndConditionsNotAccepted1() throws Exception {
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getOutput()).thenReturn("[T&C-APPROVAL-MISSING] Approval of the Terms & Conditions is required in order to use this testbed. Please visit https://example.com/termsandconditions");
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.GENIRESPONSE_REFUSED);
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(true));
    }

    @Test
    public void testIsReservationProblem() throws Exception {
        AbstractGeniAggregateManager.AggregateManagerReply reply = mock(AbstractGeniAggregateManager.AggregateManagerReply.class);
        when(reply.getOutput()).thenReturn("Refused cause mobile10 is pre-reserved.");
        when(reply.getValue()).thenReturn(new Integer(0));
        when(reply.getRawValue()).thenReturn(new Integer(0));
        when(reply.getGeniResponseCode()).thenReturn(GeniAMResponseCode.GENIRESPONSE_REFUSED);
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.NOT_ENOUGH_FREE_RESOURCES), is(false));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.RESERVATION_PROBLEM), is(true));
        assertThat(AbstractGeniAggregateManager.isCommonError(reply, AbstractGeniAggregateManager.CommonAMError.TERMS_AND_SERVICES_NOT_ACCEPTED), is(false));
    }
}